<?php
/**
 * Better HTTP Redirects
 * 
 * Copyright 2010 by hakre <hakre.wordpress.com>, some rights reserved.
 *
 * Wordpress Plugin Header:
 * 
 *   Plugin Name:    Better HTTP Redirects
 *   Plugin URI:     http://hakre.wordpress.com/plugins/better-http-redirect
 *   Description:    Better HTTP Redirects makes your Blog's redirects to play more nicely the HTTP standards. Dual Mode Plugin, can be used as a standard Plugin or as a Must-Use Plugin.
 *   Version:        1.0
 *   Stable tag:     1.0
 *   Min WP Version: 2.9
 *   Author:         hakre
 *   Author URI:     http://hakre.wordpress.com/
 *   Donate link:    http://www.prisonradio.org/donate.htm
 *   Tags:           HTTP, redirect
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

return betterRedirectsPlugin::bootstrap();

class betterRedirectsPlugin {
	/** @var string queued hypertext message */
	private $_message;
	/** @var betterRedirectsPlugin */
	static $__instance;
	final public static function bootstrap() {
		if (null==betterRedirectsPlugin::$__instance) {
			betterRedirectsPlugin::$__instance = new betterRedirectsPlugin();
		}		
		return betterRedirectsPlugin::$__instance;
	}
	private function __construct() {
		if (!function_exists('add_filter'))
			return;
		add_filter('wp_redirect_status', array($this, 'wp_redirect_status'), 10, 2);					
	}	
	/**
	 * wp_redirect_status hook
	 * 
	 * @-wp-hook wp_redirect_status 
	 * @param int $status
	 * @param string $location
	 */
	public function wp_redirect_status($status, $location) {
		$location && (3 ==(int)($status/100)) && $this->test($status, $location);
		return $status;
	}
	/**
	 * Test for Hypertext
	 * 
	 * @see wp_redirect_status()
	 */
	public function test($status, $location) {
		$status = (int) $status;
		$hypertext = $this->_hypertext($location, $status, $_SERVER['REQUEST_METHOD']);		
		$hypertext = apply_filters('redirect_hypertext', $hypertext, $location, $status);
		$this->_message = $hypertext;
	}
	/**
	 * generate a default hypertext message
	 * 
	 * handles for non-HEAD requests with the following status:
	 * 
	 * 	301 Moved Permanently
	 *  302 Found
	 *  303 See Other
	 *  307 Temporary Redirect
	 * 
	 * @param string  $location  redirect location
	 * @param int     $status    http status code
	 * @return string hypertext fragment for status code (empty string if there is none)
	 */
	private function _hypertext($location, $status, $method) {
		if ('HEAD' == $method) return '';			
		$status   = (int) $status;
		$map      = array(301 => 0, 302 => 0, 303 => 1, 307 => 1);
		if (!isset($map[$status])) return '';
		$desc     = get_status_header_desc( $status );			
		$version  = $GLOBALS['wp_version'];
		list($link, $title, $description, $charset) = array_map('get_bloginfo', array('wpurl', 'name', 'description', 'charset'));
		$homepage = sprintf('<a href="%s">%s - %s</a>', esc_attr( $link ), esc_html( $title ), esc_html( $description ) );
		$self     = array_map('esc_html', get_file_data(__FILE__, array('name' => 'Plugin Name', 'plugver' => 'Version')));		
		$extras   = vsprintf('(with %s %s)', $self);		
		$messages = array('The document has moved', 'The answer to your request is located');
		$message  = $messages[$map[$status]];
		
		ob_start();		
### Template Begin ###		
?>
<HTML><HEAD><meta http-equiv="content-type" content="text/html;charset=<?php echo $charset; ?>">
<TITLE><?php echo $status; ?> <?php echo esc_html($desc); ?></TITLE></HEAD><BODY>
<H1><?php echo $status; ?> <?php echo esc_html($desc); ?></H1>
<?php echo esc_html($message); ?>

<A HREF="<?php echo esc_attr( $location ); ?>">here</A>.
<HR>
<address>WordPress/<?php echo esc_html( $version ); ?> <?php echo $extras ?> at <?php echo $homepage; ?>.</address>
</BODY></HTML>
<?php
### Template End ###
		return ob_get_clean();
	}
	/**
	 * on shutdown, output the hypertext if available.
	 */		
	public function __destruct() {
		echo $this->_message;
	}
}

# EOF;